<?php


/**
 * The IO class handles the extraction of the
 * content.xml from the zipped *.odt file and
 * writting it back later on.
 *  
 * $Id: IO.class.php,v 1.2 2006/09/18 18:51:18 Andi Exp $
 *
 * @license    http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright  Copyright in 2006 by Andi Hotz
 * @copyright  GNU Lesser General Public License
 * @author     Andi Hotz
 * @version    $Revision: 1.2 $
 * @package    ODFLib
 * @since      0.0.1
 */
class IO {
	
    /**
     * Filename of the archive
     */
	private $archiveName;
	/**
	 * Name of the document without ending (.odt)
	 */
	private $filename;
	/**
	 * Ending of the document (.odt)
	 */
	private $fileending;
	/**
	 * Name of the xml file in the achive (content.xml)
	 */
	private $xmlfile;
	/**
	 * Path to $xmlfile in the archive
	 */
	private $xmldir;
	/**
	 * Contents of the xml file
	 */
	private $xmlcontents;
	/**
	 * Path for preview and XML contents
	 */
	private $tmpdir;
	/**
	 * Output directory for documents
	 */
	private $outputdir;
	
	/**
	 * Initialise some member variables
	 * @param name of the archive
	 */
	public function __construct($aName){
		$this->archiveName=$aName;
		$dsep = DIRECTORY_SEPARATOR;
		$scriptPath = dirname($_SERVER["SCRIPT_FILENAME"]);
		$this->outputdir = $scriptPath.$dsep.'output'.$dsep;
		
		if (!($this->outputdir = $this->checkPath($this->outputdir))) throw new Exception("Kein temporäres Verzeichnis angegeben!", 10);
		// add a slash to the directory
		if (!preg_match('/\/$/', $this->outputdir)) $this->outputdir .= '/';
		$this->tmpdir = $this->outputdir;
		// if not available make a new directory in the filesystem
		if (!is_dir($this->tmpdir)) mkdir($this->tmpdir);
	}
	
	/**
	 * Open an Archive and extract the file $fileName
	 * as a string
	 * @param  $fileName file to extract from the archive
	 * @return String with the content of $fileName
	 * @throws Exception if an error occured while opening the archive
	 */
	protected function openArchive($fileName){
		// TODO: Refactoring to work with zip from the php framework
		preg_match('/(.+)\.(\w{3,4})$/', $fileName, $_tmp);
	    $this->filename = $_tmp[1];
	    $this->fileending = $_tmp[2];
   		$this->xmlfile = $fileName;
   		$this->xmldir = '';
	    require_once('pclzip.lib.php');
		$archiv = new PclZip($this->archiveName);
		$result = $archiv->extract(PCLZIP_OPT_BY_NAME,$fileName,PCLZIP_OPT_EXTRACT_AS_STRING);
		if ($result==0) throw new Exception("Problems while unzipping:".$archiv->errorInfo(true),10);
		return $result[0]['content'];
	}
	
	/**
	 * Extracts the content.xml file from an archive
	 * @return content.xml as string
	 * @throws Exception if an error occured while opening the archive
	 * @see openArchive($archiveName, $fileName)
	 */
	public function getContent(){
		return $this->openArchive('content.xml');
	}
	
	/**
	 * Extracts the content.xml file from an archive
	 * @return styles.xml as string
	 * @throws Exception if an error occured while opening the archive
	 * @see openArchive($archiveName, $fileName)
	 */
	public function getStyle(){
		return $this->openArchive('styles.xml');
	}
	
	/**
	 * Save the content of a file into the archive
	 * @param $fileName file name in the archive
	 * @param $fileContents file as string
	 * @throws Exception if the file cannot be added or the
	 * duplicate cannot be removed
	 */
	public function saveArchive($fileName,$fileContents) {
		if (!is_dir($this->outputdir)) mkdir($this->outputdir);
	 	$archiveName_new = $this->outputdir.$this->archiveName;
	 	copy($this->archiveName,$archiveName_new);
	 	$zip = new ZipArchive;
		$res = $zip->open($archiveName_new);
		if ($res === TRUE) {
		   $zip->addFromString($fileName, $fileContents);
		   $zip->close();
		} else {
		   throw new Exception("Error: File could not be added to archive.");
		}
		/*
	 	$this->copyFile($this->archiveName,$archiveName_new);
	 	//copy($this->archiveName,$archiveName_new);
	 	file_put_contents($fileName,$fileContents);
	 	$archive = new PclZip($archiveName_new);
	 	if ($archive->delete(PCLZIP_OPT_BY_NAME,dirname($fileName).'/'.$fileName)==0){
	 		throw new Exception("Error: ".$archive->errorInfo(true));
	 	}
	 	if ($archive->add($fileName,PCLZIP_OPT_ADD_PATH,dirname($fileName))==0){
	 		throw new Exception("Error: ".$archive->errorInfo(true));
	 	}*/
	 }

	 /**
	 * Save the content of a file into the archive
	 * @param string $fileContents changed contents of the xml file
	 * @throws Exception if the file cannot be added or the
	 * duplicate cannot be removed
	 */
	 public function saveFile($fileContents) {
	      $this->xmlcontents=$fileContents;
//	      echo $this->tmpdir.", ".$this->outputdir.", ".$this->filename.", ".$this->fileending.", ".$this->xmlfile.", ".$this->xmlcontents;
		  if (!($this->tmpdir && $this->outputdir && $this->filename && $this->fileending && $this->xmlfile && $this->xmlcontents)) throw new Exception("Missing outputdir, filename oder filecontents", 65);
		  if (!file_put_contents($this->tmpdir . $this->xmlfile, $this->xmlcontents)) throw new Exception("Temp XML file could not be written", 66);
		  $_h = getDate();
		  $_erweiterung = sprintf("_%4d-%02d-%02d_%02d-%02d-%02d_", $_h['year'], $_h['mday'], $_h['mon'], $_h['hours'], $_h['minutes'], $_h['seconds']);
		  $_name_neu = $this->outputdir . $this->filename . $_erweiterung;
		  $_index = 1;
		  while (is_file($_name_neu . $_index . '.' . $this->fileending)) ++$_index;
		  $_name_neu .= $_index . '.' . $this->fileending;
		  // copy of the file
		  $this->copyFile($this->filename . '.' . $this->fileending, $_name_neu);
		  require_once('pclzip.lib.php');
		  $archiv = new PclZip($_name_neu);
		  // delete old version of file from archive
		  if ($archiv->delete(PCLZIP_OPT_BY_NAME,$this->xmlfile)==0) throw new Exception("Error on deleting from ZIP archive: " . $archiv->errorInfo(true));
		  // copy changed file from tempdir into archive
		  if ($archiv->add($this->tmpdir . $this->xmlfile, PCLZIP_OPT_REMOVE_PATH, $this->tmpdir, PCLZIP_OPT_ADD_PATH, $this->xmldir) == 0) throw new Exception("Error on writing ZIP archive: " . $archiv->errorInfo(true));
		  return($_name_neu);
	 }

 	  /**
 	   * Check if a path is valid e.g. path doesn't point to system
 	   * files.
 	   * @param string $_path path to be checked
 	   * @return true if path is valid
 	   * @throws $_path is not a valid argument
 	   */
	  protected function checkPath($_path) {
  		  if (!$_path) return false;
  		  // Change backslashes to slashes
  		  $_path = str_replace(array('\\','/'), '/', $_path);
  		  if (preg_match('/\.\.\//', $_path) || preg_match('/^\//', $_path)) throw new Exception("Argument $_path contains non valid path", 30);
  		  if (!preg_match('/\w/', $_path)) return false;
  		 return $_path;
 	  }

 	  /**
 	   * Copy a file
 	   * @param string $_filename path of source file
 	   * @param string $_filename_new path of destination file
 	   * @throws Copying failed
 	   */
   	  protected function copyFile($_filename, $_filename_new) {
  		  foreach (array($_filename, $_filename_new) as $_path) $this->checkPath($_path);
echo "copy from ".$_filename." to ". $_filename_new."<br/>";
  		  if (!copy($_filename, $_filename_new)) throw new Exception("Copiing of file $_filename to $_filename_new failed", 70);
 	  }

	 // Hilfsfunktion: Liest Dokument aus Zip-Archiv
	 // 1. Parameter: Archivname
	 // 2. Parameter: auszulesendes Dokument
	 // 3. Parameter: Verzeichnis, in welches das Dokument geschrieben wird;
	 // ist der 3. Parameter leer, gibt die Funktion nur den Dateiinhalt zurück
	 // nutzt: $this->checkPath() und PclZip
	 public function oeffnen($_archivname, $_filename/*, $_schreibpfad*/) {
		// TODO: Refactoring to work with zip from the php framework
	 	preg_match('/(.+)\.(\w{3,4})$/', $_archivname, $_tmp);
	    $this->filename = $_tmp[1];
	    $this->fileending = $_tmp[2];
   		$this->xmlfile = $_filename;
   		$this->xmldir = '';
	 	foreach (array($_archivname, $_filename) as $_para) {
	   $_path = $this->checkPath($_para);
	   if (!$_para) throw new Exception("Archive missing", 20);
	  }
	  //$_schreibpfad = $this->checkPath($_schreibpfad);
	  if (!file_exists($_archivname)) throw new Exception("$_archivname could not be found", 21);
	  // bindet PclZip ein
	  require_once('pclzip.lib.php');
	  $archiv = new PclZip($_archivname);
	  // kein Schreibpfad -> Dokument in String entpacken
	  //if (!$_schreibpfad) {
	   $_ret = $archiv->extract(PCLZIP_OPT_BY_NAME, $_filename, PCLZIP_OPT_EXTRACT_AS_STRING);
	  // ansonsten Pfad der zu extrahierenden Datei vom Dateinamen abtrennen
	  /*} else {
	   preg_match('/^(.+)\/(.+)$/', $_filename, $_archivpfad);
	   // Doubletten löschen - PclZip überschreibt Dateien nicht zuverlässig
	   if (file_exists($_schreibpfad . $_archivpfad[2])) unlink($_schreibpfad . $_archivpfad[2]);
	   // in Datei entpacken
	   $_ret = $archiv->extract(PCLZIP_OPT_BY_NAME, $_filename, PCLZIP_OPT_PATH, $_schreibpfad, PCLZIP_OPT_REMOVE_PATH, $_archivpfad[1]);
	  }*/
	  // Rückgabewert 0 = Fehler
	  if ($_ret == 0) throw new Exception("Problems while unzipping: " . $archiv->errorInfo(true), 22);
	  // gibt den Dateinamen zurück, falls eine Datei geschrieben wurde,
	  // ansonsten den Inhalt
	  // $_ret[0] ist ein benannter Array, der das gezippte Dokument enthält
	  return /*$_schreibpfad? $_ret[0]['filename'] :*/ $_ret[0]['content'];
	 } 	  
}
?>