<?php

/**
 * This class handles the adding of styled and unstyled
 * tables in the DOM structure. A Table must then be filled with
 * TableRows
 *  
 * $Id: Table.class.php,v 1.1.1.1 2006/09/17 15:41:38 Andi Exp $
 *
 * @license    http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright  Copyright in 2006 by Andi Hotz
 * @copyright  GNU Lesser General Public License
 * @author     Andi Hotz
 * @version    $Revision: 1.1.1.1 $
 * @package    ODFLib
 * @subpackage odfNodes
 * @since      0.0.1
 */
class Table{
	/**
	 * DOMDocument
	 */
	private $dom;
	
	/**
	 * Name of the table
	 */
	private $tableName;
	
	/**
	 * Number of Columns
	 */
	private $numOfCols;
	
	/**
	 * Initialise the DOMDocument.
	 * The table name is initialised with a timestamp
	 * @param DOMDocument $dom
	 */
	public function __construct(DOMDocument $dom, $numOfCols = 1){
		$this->dom=$dom;
		$this->tableName="Table".time();
		$this->numOfCols=$numOfCols;
	}
	
	/**
	 * Add a new table node to the DOM structure
	 *
	 * @param DOMNode $afterNode table is appended after
	 * @return the new table node
	 * 	 */
	public function addTable(DOMNode $afterNode){
		$table = $this->dom->createElement('table:table');
		$colDef = $this->dom->createElement('table:table-column');
		$colDef->setAttribute('table:number-columns-repeated',$this->numOfCols);
		$table->appendChild($colDef);
		require_once('Replace.class.php');
		return Replace::insertAfter($this->dom,$afterNode,$table);
	}

	/**
	 * Add a new table node to the DOM structure
	 *
	 * @param DOMNode $afterNode table is appended after
	 * @param string $style of the table
	 * @return the new table node
	 */
	public function addStyledTable(DOMNode $afterNode,$style){
		$table = $this->dom->createElement('table:table');
		$table->setAttribute('table:style-name', $style);
		$colDef = $this->dom->createElement('table:table-column');
		$colDef->setAttribute('table:number-columns-repeated',$this->numOfCols);
		$table->appendChild($colDef);
		require_once('Replace.class.php');
		return Replace::insertAfter($this->dom,$afterNode,$table);
	}
	
}
?>