<?php

/**
 * This class handles the adding 
 * tablecells in the DOM structure.
 *  
 * $Id: TableCell.class.php,v 1.1.1.1 2006/09/17 15:41:38 Andi Exp $
 *
 * @license    http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright  Copyright in 2006 by Andi Hotz
 * @copyright  GNU Lesser General Public License
 * @author     Andi Hotz
 * @version    $Revision: 1.1.1.1 $
 * @package    ODFLib
 * @subpackage odfNodes
 * @since      0.0.1
 */
class TableCell{
	/**
	 * DOMDocument
	 */
	private $dom;
	
	/**
	 * Initialise the DOMDocument.
	 * @param DOMDocument $dom
	 */
	public function __construct(DOMDocument $dom){
		$this->dom=$dom;
	}

	/**
	 * Add a new table cell
	 * after $afterNode
	 * @param DOMNode $rowNode parent node (row-node) where
	 * the new node is added in the DOM structure as last child
	 * @param string $dataType datatype of the cell contents
	 * @param string $text content of the cell
	 * @return the new table node
	 */
	public function addTableCell(DOMNode $rowNode, $dataType='string', $text=''){
		$cell = $this->dom->createElement('table:table-cell');
		$cell->setAttribute('office:value-type',$dataType);
		$inner = $this->dom->createelement('text:p',$text);
		$cell->appendChild($inner);
		$rowNode->appendChild($cell);
	}

	/**
	 * Add a new table cell
	 * after $afterNode
	 * @param DOMNode $rowNode parent node (row-node) where
	 * the new node is added in the DOM structure as last child
	 * @param string $dataType datatype of the cell contents
	 * @param string $text content of the cell
	 * @return the new table node
	 */
	public function addStyledTableCell(DOMNode $rowNode, $dataType='string', $text='',$cellStyle,$textStyle){
		$cell = $this->dom->createElement('table:table-cell');
		$cell->setAttribute('office:value-type',$dataType);
		$cell->setAttribute('table:style-name', $cellStyle);
		$inner = $this->dom->createelement('text:p',$text);
		$inner->setAttribute('text:style-name', $textStyle);
		$cell->appendChild($inner);
		$rowNode->appendChild($cell);
	}
	
}
?>